/* -----------------------------------------------------------------------------------
                            st77xx_display.h

     Header Softwaremodul fuer farbige TFT-Displays

     unterstuetzte Displaycontroller:
     SPI
     -----------------------------------
           ili9163
           ili9340
           st7735r
           s6d02a1
           ili9225
           st7789

     MCU   :  CH32V003
     Takt  :

     27.03.2025  R. Seelig
   ----------------------------------------------------------------------------------- */

#ifndef in_st77xx_display
  #define in_st77xx_display

  #include <stdint.h>
  #include <stdlib.h>

  #include "ch32fun.h"
  #include "ch32v003_gpio.h"


  /* ------------------------------------------------------------------
                   Anschluss Display an Controller
     ------------------------------------------------------------------ */
/*
                   CH32V003F4P6

                 +--------------+
        PD4 / a7 | 1         20 | PD3 / a4
  PD5 / a5 / utx | 2    C    19 | PD2 / a3
  PD6 / a6 / urx | 3    H    18 | PD1 / swio
      nrst / PD7 | 4    3    17 | PC7 / miso
 PA1 / osc0 / a0 | 5    2    16 | PC6 / mosi
 PA2 / osc1 / a1 | 6    V    15 | PC5 / sck / scl
             gnd | 7    0    14 | PC4 / a2
             PD0 | 8    0    13 | PC3
             Vdd | 9    3    12 | PC2 / scl
             PC0 | 10        11 | PC1 / sda
                 +--------------+
*/

/* ##################################################################################
   ##################################################################################

                               Benutzereinstellungen
                            
   ##################################################################################
   ################################################################################## */

/*
  Anschluss Display an den CH32V003
  Mosi und Sck sind aufgrund der Verwendung von Hardware-SPI fest vorgegeben und
  koennen nicht geaendert werden. Die anderen Anschluesse sind frei waehlbar
   
  CH32V003        ---->   SPI-Display
  --------------------------------------------------------------------------
  PC6-mosi   16   ---->   sda / sdi / A0 / din
  PC5-sck    15   ---->   sck
  PC4        14   ---->   dc
  PC3        13   ---->   ce / cs   (kann auch direkt auf GND gelegt werden)
  PC2        12   ---->   rst

*/
  // PC5 ist immer SPI-Clock (Hardware-SPI)
  // PC6 ist immer SPI-MOSI (Hardware-SPI)

  #define lcd_ce_init()        PC3_output_init()
  #define lcd_dc_init()        PC4_output_init()
  #define lcd_rst_init()       PC2_output_init()
  #define lcd_pin_init()       { lcd_ce_init(); lcd_dc_init(); lcd_rst_init(); }

  #define dc_set()             PC4_set()
  #define dc_clr()             PC4_clr()

  #define ce_set()             PC3_set()
  #define ce_clr()             PC3_clr()

  #define rst_set()            PC2_set()
  #define rst_clr()            PC2_clr()

  #define lcd_enable()         (ce_clr())
  #define lcd_disable()        (ce_set())

  /* ------------------------------------------------------------------
                            Displayauswahl

       es kann (und muss) nur ein einziges Display ausgewaehlt sein.
     ------------------------------------------------------------------ */

  // ----------------------- SPI- Displays ----------------------------

  #define  ili9163                  0
  #define  ili9340                  0
  #define  st7735r                  1
  #define  s6d02a1                  0
  #define  ili9225                  0
  #define  st7789                   0

  // -----------------  ST7735R, 2. Generation Controller ------------
  //  die 128er Display der 2. Generation haben in Verbindung mit
  //  ST7735 eine andere StartColum und RowColum

  #define  st7735r_g2               1
  
  /*  ------------------------------------------------------------
                         Displayaufloesung
      ------------------------------------------------------------ */

  #define _xres                   128
  #define _yres                   160

  #define mirror                  0                 // 0 : normale Ausgabe
                                                    // 1 : Spiegelbildausgabe  

  /*  ------------------------------------------------------------
                       Setupflags fuer SPI-Displays
      ------------------------------------------------------------ */

  // fuer Berechnung der Adressen des Video-Rams. ACHTUNG: manche Chinadisplays 
  // behandeln 128x128 Displays so, als haette es 160 Pixel in Y-Aufloesung.
  // Fuer diesen Fall kann in "tft128" angegeben werden um welches Display es
  // sich handelt. Hat man hier den falschen Typ gewählt, ist das auf dem
  // Display durch einen breiten Streifen mit rauschenden Bildpunkten zu
  // erkennen. tft128 hat keine Auswirkung auf Displays mit anderer Aufloesung
  // als 128x128

  #define tft128                  2                 // 1: Display ohne Offset (aelter)
                                                    // 2: Display mit Offset (neuer)

  #define rgbseq                  0                 // Reihenfolge der erwarteten Farbuebergabe bei ST7735 LC-Controllern
                                                    // 0: blau-gruen-rot
                                                    // 1: rot-gruen-blau

  #define tft_wait                0                 // 0 = keine Wartefunktion nach spi_out
                                                    // 1 = es wird nach spi_out ein nop eingefuegt


  #define flickerreduce           0                 // 0 : normal
                                                    // 1 : Reduzierung fuer neuere KMR-1.8 SPI Displays

  #define negativout              0                 // 0 : normal
                                                    // 1 : Farben werden invertiert wiedergegeben
                                                    

  /* ------------------------------------------------------------------
        verfuegbare Textfonts auswaehlen (auch Kombinationen erlaubt)
     ------------------------------------------------------------------ */

  #define fnt5x7_enable             0                // 1 : Font verfuegbar
                                                     // 0 : nicht verfuegbar
  #define fnt8x8_enable             1                // 1 : Font verfuegbar
                                                     // 0 : nicht verfuegbar
  #define fnt12x16_enable           0                // 1 : Font verfuegbar
                                                     // 0 : nicht verfuegbar

  #define lastascii 126                              // letztes verfuegbares Asciizeichen
                                                    
                                                    
/* ##################################################################################
   ##################################################################################

                            Ende der Benutzereinstellungen
                          ab hier keine 
                            
   ##################################################################################
   ################################################################################## */
                                                    

  /*  ------------------------------------------------------------
        Sonderfall TFT 128x128 / ST7735 Controller 2. Generation
      ------------------------------------------------------------ */
  #if ((st7735r == 1) && (st7735r_g2 == 1) && (_xres == 128) && (_yres == 128))
    #define colofs                2
    #define rowofs                3
  #else
    #define colofs                0
    #define rowofs                0
  #endif

  /*  ------------------------------------------------------------
        Display-Offset neue/alte 128x128 Pixel TFTs
      ------------------------------------------------------------ */
  #if (tft128 == 2)
    #define _lcyofs               -32               // manche Display sprechen das Display an
                                                    // als haette es 160 Pixel Y-Aufloesung
  #else
    #define _lcyofs               0
  #endif

  /*  ------------------------------------------------------------
       soll innerhalb der fillrect - Funktion ein Fastfill mittels
       der Funktionen des Displays vorgenommen werden (hier
       funktioniert dann ein "Drehen" mittels outmode NICHT)
     ------------------------------------------------------------- */

  #define  fastfillmode             0

  /*  ------------------------------------------------------------
                         P R O T O T Y P E N
      ------------------------------------------------------------ */

  // ----------------- LCD - Benutzerfunktionen ---------------

  void lcd_init(void);                                                        // initialisiert Display
  void lcd_orientation (uint8_t ori);                                         // kompletten Displayinhalt bei der Ausgabe drehen
  void putpixel(int x, int y,uint16_t color);                                 // schreibt einen einzelnen Punkt auf das Display
  void clrscr();                                                              // loescht Display-Inhalt
  void fastxline(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t color);      // zeichnet eine Linie in X-Achse
  void fillrect(int x1, int y1, int x2, int y2, uint16_t color);              // fuellt einen rechteckigen Bereich mit Farbe aus
  uint16_t rgbfromvalue(uint8_t r, uint8_t g, uint8_t b);                     // konvertiert einen 24 Bit RGB-Farbwert in einen 16 Bit Farbwert
  uint16_t rgbfromega(uint8_t entry);                                         // konvertiert einen Farbwert aus der EGA-Palette in einen 16 Bit Farbwert
  void gotoxy(unsigned char x, unsigned char y);                              // setzt den Textcursor fuer Textausgaben
  void setfont(uint8_t nr);                                                   // setzt Schriftstil: 0= 8x8 Pixel, 2= 5x7 Pixel
  void lcd_putchar(char ch);                                                  // setzt ein Zeichen auf das Display
  void lcd_putchar5x7(unsigned char ch);
  void lcd_putchar8x8(unsigned char ch);
  void lcd_putchar12x16(unsigned char ch);
  void putcharxy(int oldx, int oldy, unsigned char ch);                       // setzt ein Zeichen an der angegebenen  Grafikkoordinate
  void outtextxy(int x, int y, uint8_t dir, char *dataPtr);                   // gibt einen String an Pixelkoordinaten auf dem LCD aus
  void line(int x0, int y0, int x1, int y1, uint16_t color);                  // zeichnet eine Linie
  void rectangle(int x1, int y1, int x2, int y2, uint16_t color);             // zeichnet ein Rechteck
  void ellipse(int xm, int ym, int a, int b, uint16_t color );                // zeichnet eine Ellipse
  void fillellipse(int xm, int ym, int a, int b, uint16_t color );            // zeichnet eine ausgefuellte Ellipse
  void circle(int x, int y, int r, uint16_t color );                          // zeichnet einen Kreis
  void fillcircle(int x, int y, int r, uint16_t color );                      // zeichnet einen ausgefuellten Kreis
  void showimage(uint16_t ox, uint16_t oy, const unsigned char* const image, uint16_t fwert);     // zeichnet ein monochromes Bitmap
  void lcd_puts(char *c);                                                     // schreibt einen Textstring auf das LCD
  void turtle_moveto(int x, int y);
  void turtle_lineto(int x, int y, uint16_t col);

  // ---------------- Low-level Displayfunktionen -------------

  void set_ram_address (uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);  // setzt den zu beschreibenden Speicherbereich
  void setcol(int startcol);                    // setzt zu beschreibende Spalte
  void setpage(int startpage);                  // setzt zu beschreibende Reihe
  void setxypos(int x, int y);                  // setzt die zu beschreibende Koordinate im Display-Ram

  // --------------------- SPI-Funktionen ---------------------

  void spi_init(void);
  void spi_out(uint8_t data);
  void wrcmd(uint8_t cmd);                      // schreibt einzelnes Kommandodatum (Registerzugriff)
  void wrdata(uint8_t data);                    // schreibt einzelnen Registerwert oder Ramwert
  void wrdata16(int data);                      // schreibt einen Integerwert

  /*  ------------------------------------------------------------
                      EGA - Farbzuweisungen
      ------------------------------------------------------------ */

  #define black                   0
  #define blue                    1
  #define green                   2
  #define cyan                    3
  #define red                     4
  #define magenta                 5
  #define brown                   6
  #define grey                    7
  #define darkgrey                8
  #define lightblue               9
  #define lightgreen              10
  #define lightcyan               11
  #define lightred                12
  #define lightmagenta            13
  #define yellow                  14
  #define white                   15

  //-------------------------------------------------------------
  // Registerzuordnung der Adressierungsregister der
  // verschiedenen Displaycontroller
  //-------------------------------------------------------------


  #if  (ili9225 == 1)
    #define coladdr      0x20
    #define rowaddr      0x21
    #define writereg     0x22
  #else
    #define coladdr      0x2a
    #define rowaddr      0x2b
    #define writereg     0x2c
  #endif

  //-------------------------------------------------------------
  //  Variable Farben
  //-------------------------------------------------------------

  extern uint16_t textcolor;        // Beinhaltet die Farbwahl fuer die Vordergrundfarbe
  extern uint16_t bkcolor;          // dto. fuer die Hintergrundfarbe
  extern uint16_t egapalette [];    // Farbwerte der DOS EGA/VGA Farben


  //-------------------------------------------------------------
  //  Variable und Defines Schriftzeichen
  //-------------------------------------------------------------

  extern int aktxp;                 // Beinhaltet die aktuelle Position des Textcursors in X-Achse
  extern int aktyp;                 // dto. fuer die Y-Achse
  extern uint8_t outmode;           // Richtungssinn der Displayausgabe
  extern uint8_t textsize;          // Skalierung der Ausgabeschriftgroesse
  extern uint8_t fntfilled;         // gibt an, ob eine Zeichenausgabe ueber einen Hintergrund gelegt
                                    // wird, oder ob es mit der Hintergrundfarbe aufgefuellt wird
                                    // 1 = Hintergrundfarbe wird gesetzt, 0 = es wird nur das Fontbitmap
                                    // gesetzt, der Hintergrund wird belassen

  enum {fnt8x8, fnt12x16, fnt5x7};

  extern uint8_t fontnr;            // 0= 8x8 Pixel, 1, 12x16, 2= 5x7 Pixel
  extern uint8_t fontsizex;
  extern uint8_t fontsizey;

  #define PSTR(txt)   ((char*)txt)  // uebergibt einen Zeiger auf einen Text (fuer outtextxy)

  //-------------------------------------------------------------
  //  Variable Turtle-Grafik
  //-------------------------------------------------------------
  extern int t_lastx, t_lasty;       // x,y - Positionen der letzten Zeichenaktion von moveto


#endif
