/* -------------------------------------------------------
                       glcd_demo_v2.c

     Demoprogramm fuer grafische TFT-Displays mit Auf-
     loessungen von 128x128 oder 160x128 Pixeln.

     Hinweis: Funktioniert nur mit CH32V003F4P6, da die
     Chips in den 8 und 16 poligen Gehaeusen keinen
     SPI-SCK Anschluss nach aussen gefuehrt haben.

     MCU   :  CH32V003
     Takt  :

     02.06.2025  R. Seelig
   ------------------------------------------------------ */

#include <stdint.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "ch32fun.h"
#include "ch32v003_gpio.h"

#include "st77xx_display_v2.h"

// Aufloesung fuer Liniendiagramm
#define y_aufloes          128


// Bilddateien
#include "./bmp/swmanga01.h"

/* --------------------------------------------------------------
     Lookup-Tabelle fuer Sinuswinkel (DEG-Mode) von 0..90 Grad
     multipliziert mit 1000 => Integerwerte mit 3 Nachkomma-
     stellen.  Schrittweite Arrayelemente betraegt 1 Grad
   -------------------------------------------------------------- */
int16_t sindeg_lookup[91] =
{
     0,  17,  34,  52,  69,  87, 104, 121,
   139, 156, 173, 190, 207, 224, 241, 258,
   275, 292, 309, 325, 342, 358, 374, 390,
   406, 422, 438, 453, 469, 484, 500, 515,
   529, 544, 559, 573, 587, 601, 615, 629,
   642, 656, 669, 681, 694, 707, 719, 731,
   743, 754, 766, 777, 788, 798, 809, 819,
   829, 838, 848, 857, 866, 874, 882, 891,
   898, 906, 913, 920, 927, 933, 939, 945,
   951, 956, 961, 965, 970, 974, 978, 981,
   984, 987, 990, 992, 994, 996, 997, 998,
   999, 999, 1000
};


/* --------------------------------------------------------
                          my_putchar

   wird von my-printf / printf aufgerufen und hier muss
   eine Zeichenausgabefunktion angegeben sein, soll
   my_printf verwendet werden !

   my_putchar kann jedoch, wie in diesem Beispiel hier,
   natuerlich auch von anderen Funktionen aufgerufen werden,
   um eine Umschaltung von Ausgabekanaelen flexibler zu
   gestalten.
   -------------------------------------------------------- */
void my_putchar(char ch)
{
  lcd_putchar(ch);
}

/* ------------------------------------------------------------
                          xline_dotted

     zeichnet eine gepunktete Linie in X-Achse

     Uebergabe:
          x1 :     X-Position ab der die Linie gezeichnet wird
          y1 :     Y-Position der zu zeichnenden Linie
          x2 :     X-Position zu der die Linie gezeichnet wird
          lcol :   Zeichenfarbe der Dots der Line
          bcol :   Farbe an der keine Dots gezeichnet werden
          muster : Bitmuster, mit der die Dots der Linie ge-
                   zeichnet werden
   ------------------------------------------------------------ */
void xline_dotted(int x1, int y1, int x2, uint16_t lcol, uint16_t bcol, uint16_t muster)
{
  int      xi;
  uint16_t mask;

  mask= 0x8000;
  for (xi= x1; xi < x2; xi++)
  {
    if (muster & mask) { putpixel(xi, y1, lcol); }
                  else { putpixel(xi, y1, bcol); }
    mask = mask >> 1;
    if (!mask) { mask= 0x8000; }
  }
}

/* ------------------------------------------------------------
                          yline_dotted

     zeichnet eine gepunktete Linie in Y-Achse

     Uebergabe:
          x1 :     X-Position der zu zeichnenden Linie
          y1 :     Y-Position ab der die Linie gezeichnet wird
          x2 :     Y-Position zu der die Linie gezeichnet wird
          lcol :   Zeichenfarbe der Dots der Line
          bcol :   Farbe an der keine Dots gezeichnet werden
          muster : Bitmuster, mit der die Dots der Linie ge-
                   zeichnet werden
   ------------------------------------------------------------ */
void yline_dotted(int x1, int y1, int y2, uint16_t lcol, uint16_t bcol, uint16_t muster)
{
  int      yi;
  uint16_t mask;

  mask= 0x8000;
  for (yi= y1; yi < y2; yi++)
  {
    if (muster & mask) { putpixel(x1, yi, lcol); }
                  else { putpixel(x1, yi, bcol); }
    mask = mask >> 1;
    if (!mask) { mask= 0x8000; }
  }
}

/* ----------------------------------------------------------
                          sindeg

     liefert fuer einen Sinuswinkel (DEG-Mode) den Sinuswert
     multipliziert mit der Aufloesung y-Achse

     Uebergabe:

         w    : Winkel
         aufl : Aufloesung in Y-Achse

     Rueckgabe:
         Sinuswert

     Bsp.:  x= sindeg(30, 64);

            x== 32   (int16_t)(0.5 * 64)
   ---------------------------------------------------------- */
int sindeg(int w, int aufl)
{
  int sign= 1;

  w= w % 360;

  if (w> 180)
  {
    sign= -1;
    w -= 180;
  }
  if (w> 90) { w= 180-w; }
  return (sindeg_lookup[w] * aufl * sign) / 1000;
}

/* ----------------------------------------------------------
   Umrechnung des Sinuswertes in Y-Koordinate des Diagramms
   ---------------------------------------------------------- */
#define sindispy(wi)       ( (sindeg(wi, (y_aufloes / 2)) * (-1)) + (y_aufloes / 2))

/* ----------------------------------------------------------
                         diagram_draw

     zeichnet ein Diagramm mit der Zeichenfarbe in
     textcolor und der "Nichtzeichenfarbe" in bkcolor
   ---------------------------------------------------------- */
void diagram_draw(void)
{
  int i;

  for (i= 16; i< 128; i+= 16)
  {
    xline_dotted(0,i,127, textcolor, bkcolor, 0x8888);
    yline_dotted(i,0,127, textcolor, bkcolor, 0x8888);
  }
  line(64,0,64,127, textcolor);
  line(0,64,127,64, textcolor);

}

/* --------------------------------------------------------
                            main
   -------------------------------------------------------- */
int main(void)
{
  int x, y, i;
  int speed   = 20;

  SystemInit();

  lcd_init();
  lcd_orientation(0);

  while(1)
  {

    setfont(fnt8x8);
    textcolor= rgbfromvalue(0x00,0xff,0x00);
    bkcolor= rgbfromvalue(0x00,0x22,0x00);
    clrscr();

    diagram_draw();

    // Sinuskurve zeichnen
    turtle_moveto(1,sindispy(0) );
    for (i= 1; i< 122; i++)
    {
      turtle_lineto(i, sindispy( (i*3)), rgbfromega(yellow));
    }

    // Cosinuskurve zeichnen
    turtle_moveto(1,sindispy(60));
    for (i= 1; i< 122; i++)
    {
      turtle_lineto(i, sindispy( 60+(i*3)), rgbfromega(lightblue));
    }
    textcolor= rgbfromega(yellow);
    gotoxy(9,1); lcd_puts("Sinus");
    textcolor= rgbfromega(lightblue);
    gotoxy(9,2); lcd_puts("Cosinus");
    delay(3000);

    // ASCII Zeichen ausgeben
    textcolor= rgbfromega(14);
    bkcolor= rgbfromvalue(0x00,0x00,0x66);
    clrscr();
    gotoxy(2,1); lcd_puts("  CH32V003\n\r");
    gotoxy(2,2); lcd_puts("    48MHz\n\r");
    gotoxy(2,3); lcd_puts("-------------\n\r");
    gotoxy(2,4); lcd_puts("Ascii-Zeichen\n\r");

    for (x= 32; x< 128; x++)
    {
      if ((x % 16)== 0) lcd_puts("\n\r");
      my_putchar(x);
    }
    delay(3000);

    bkcolor= rgbfromvalue(0xa8, 0x79, 0x1c);
    clrscr();

    // sw-Pixelgrafik anzeigen
    showimage(0,0,&girl01bmp[0],rgbfromvalue(40,0,0));
    textcolor= rgbfromega(15);
    gotoxy(1,14); lcd_puts(" Pixelgrafik ");
    delay(3000);

    // rotating circles
    setfont(fnt8x8);
    textcolor= rgbfromvalue(0xff,0xff,0xff);
    bkcolor= rgbfromvalue(0x00,0x00,0x40);
    clrscr();
    rectangle(15,15,113,113,rgbfromega(lightblue));
    gotoxy(4,6); lcd_puts("Rotating");
    gotoxy(4,8); lcd_puts("Circles");
    for (i= 0; i< 2; i++)
    {
      for (x= 9; x< 118; x+= 3)
      {
        circle(x,9,4,rgbfromvalue(0xff,0xff,0x00));
        delay(speed);
        circle(x,9,4,rgbfromvalue(0x00,0x00,0x40));
      }

      for (y= 9; y< 118; y+= 3)
      {
        circle(121,y,4,rgbfromvalue(0xff,0xff,0x00));
        delay(speed);
        circle(121,y,4,rgbfromvalue(0x00,0x00,0x40));
      }

      for (x= 121; x> 9; x-= 3)
      {
        circle(x,121,4,rgbfromvalue(0xff,0xff,0x00));
        delay(speed);
        circle(x,121,4,rgbfromvalue(0x00,0x00,0x40));
      }

      for (y= 121; y> 9; y-= 3)
      {
        circle(9,y,4,rgbfromvalue(0xff,0xff,0x00));
        delay(speed);
        circle(9,y,4,rgbfromvalue(0x00,0x00,0x40));
      }
    }
    circle(9,y,4,rgbfromvalue(0xff,0xff,0x00));
    delay(3000);

    // Schriftgroessen anzeigen
    bkcolor= rgbfromega(white);
    textcolor= rgbfromega(green);
    clrscr();
    setfont(fnt8x8);
    gotoxy(0,1); lcd_puts("Schriftgroessen");
    line(0,18,127,18,rgbfromega(green));
    setfont(fnt12x16);
    textcolor= rgbfromega(red);
    gotoxy(1,2); lcd_puts("FNT-12x16");
    setfont(fnt8x8);
    textcolor= rgbfromega(blue);
    gotoxy(3,7); lcd_puts("FNT-8x8");
    textcolor= rgbfromega(black);
    setfont(fnt5x7);    
    gotoxy(6,11); lcd_puts("FNT-5x7");
    textsize= 0;
    delay(2000);
  }
}
